# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2020-09-07 15:11
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tiers', '0004_auto_20170321_2011'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tier',
            name='name',
            field=models.CharField(choices=[('trial', 'Trial'), ('basic', 'Basic'), ('pro', 'Professional'), ('premium', 'Premium')], default='trial', max_length=255),
        ),
    ]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion

from ..app_settings import ORGANIZATION_MODEL


class Migration(migrations.Migration):

    dependencies = [
        ('tiers', '0002_auto_20170321_1856'),
    ]

    operations = [
        migrations.AlterField(
            model_name='tier',
            name='organization',
            field=models.OneToOneField(related_name='tier', null=True, on_delete=django.db.models.deletion.SET_NULL, blank=True, to=ORGANIZATION_MODEL),
        ),
    ]



# -*- coding: utf-8 -*-
from django.contrib import admin

from .models import Tier


def make_exempt(modeladmin, request, queryset):
    queryset.update(tier_enforcement_exempt=True)


make_exempt.short_description = "Exempt from tier enforcement"


class TierAdmin(admin.ModelAdmin):
    list_display = ('organization', 'get_microsites', 'name', 'tier_expires_at',
                    'tier_enforcement_grace_period', 'tier_enforcement_exempt',)
    search_fields = ['organization__name', 'name']
    list_filter = ['name', 'tier_expires_at']
    actions = [make_exempt]

    def get_microsites(self, obj):
        return "\n".join(obj.organization.microsites.values_list('subdomain', flat=True))
    get_microsites.short_description = 'Microsite'
    get_microsites.admin_order_field = 'organization__microsites__subdomain'


admin.site.register(Tier, TierAdmin)



from datetime import timedelta
from functools import wraps

from django.utils import timezone
from django.utils.timesince import timeuntil
from django.db import models

from .app_settings import ORGANIZATION_MODEL

from model_utils.models import TimeStampedModel
from model_utils import Choices


def set_default_expiration():
    return timezone.now() + timedelta(days=30)


def check_if_exempt(f):
    @wraps(f)
    def wrapper(self, **kwargs):
        if self.tier_enforcement_exempt:
            return False
        return f(self, **kwargs)
    return wrapper


class Tier(TimeStampedModel):
    TIERS = Choices(
        ('trial', 'TRIAL', 'Trial'),  # Expires in 30 days
        ('basic', 'BASIC', 'Basic'),
        ('pro', 'PRO', 'Professional'),
        ('premium', 'PREMIUM', 'Premium'),
    )

    name = models.CharField(
        max_length=255,
        choices=TIERS,
        default=TIERS.TRIAL)
    organization = models.OneToOneField(
        ORGANIZATION_MODEL,
        related_name='tier',
        null=True,
        blank=True,
        on_delete=models.DO_NOTHING)
    tier_enforcement_exempt = models.BooleanField(default=False)
    tier_enforcement_grace_period = models.PositiveIntegerField(default=14)
    tier_expires_at = models.DateTimeField(
        default=set_default_expiration)

    def __unicode__(self):
        return u"{0} - {1}".format(self.organization.name, self.name)

    @check_if_exempt
    def has_tier_expired(self):
        """Helper function that checks whether a tier has expired"""
        return timezone.now() > self.tier_expires_at

    @check_if_exempt
    def has_tier_grace_period_expired(self):
        """Helper function that checks whether a tier's grace period has expired"""
        return (
            timezone.now() >
            (self.tier_expires_at + timedelta(days=self.tier_enforcement_grace_period)))

    @check_if_exempt
    def time_til_tier_expires(self, now=None):
        """Pretty prints time left til expiration"""
        return timeuntil(self.tier_expires_at, now)



#!/usr/bin/env python

from setuptools import setup, find_packages

import tiers

setup(
    name='django-tiers',
    version=tiers.__version__,
    description='Django tier management',
    long_description=open('README.md').read(),
    author='Appsembler',
    url='https://github.com/appsembler/django-tiers',
    license='MIT',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ],
    packages=find_packages(exclude=['tests', 'fake_organizations']),
    install_requires=[
        'django<3',
        'django-model-utils<=2.3.1',
        'python-dateutil<=2.8.1',
    ],
)
